# Copyright (c) 2010 Outpost Embedded, LLC
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.  You should have
# received a copy of the GNU General Public License along with this program; if
# not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
# Floor, Boston, MA 02110-1301, USA. Any Outpost Embedded trademarks that are
# incorporated in the source code or documentation are not subject to the GNU
# General Public License and may only be used or replicated with the express
# permission of Outpost Embedded, LLC.

import os, sys, tempfile, logging
from optparse import OptionParser

from rapidwriter import __version__ as rapidwriter_version
from rapidwriter.logger import configure_logging


def get_default_log_file():
    if (
      sys.platform == 'win32'
      and os.path.basename(sys.executable) not in ('python.exe', 'pythonw.exe')
    ):
        # rapidwriter.exe -- write the log file to
        # e.g. C:\Program Files\RapidWriter\rapidwriter.log
        dirpath = os.path.dirname(sys.executable)
        filename = 'rapidwriter.log'
    else:
        hd_sym = '~'
        dirpath = os.path.expanduser(hd_sym)
        if dirpath == hd_sym:
            # User has no home directory?
            dirpath = tempfile.gettempdir()
        filename = '.rapidwriter.log'
    return os.path.join(dirpath, filename)


def main(argv):
    parser = OptionParser()

    # FIXME: --log=- doesn't make any sense with rapidwriter.exe.

    parser.add_option(
      '--log',
      default = get_default_log_file(),
      help = (
        "log file; use - for stderr [default %default]"
      ),
    )
    parser.add_option(
      '--log-level',
      default = 'debug',
      help = 'debug, info, warning, or error [default %default]',
    )

    opts, args = parser.parse_args(argv[1:])

    configure_logging(opts.log, opts.log_level)

    # FIXME: Redirect stderr, stdout to log file.  In particular,
    # rapidwriter.exe creates an additional log file if anything gets written
    # to stderr.

    logger = logging.getLogger()

    logger.info('RapidWriter version %r', rapidwriter_version)

    try:
        from PyQt4 import QtGui

        app = QtGui.QApplication(argv)

        if sys.platform == 'win32':
            from rapidwriter.backends.windows import WindowsBackend as backend_cls
        else:
            from rapidwriter.backends.linux import LinuxBackend as backend_cls

        from rapidwriter.gui import RapidWriterDialog

        dialog = RapidWriterDialog(backend_cls)
        dialog.exec_()

    except KeyboardInterrupt:
        pass

    except Exception:
        logger.exception('fatal exception')
        return 1

    return 0
